@@ -1,12 +1,31 @@
 #![feature(box_into_inner)]
 extern crate proc_macro;
 
+#[cfg(test)]
+mod function_tests;
+
+use convert_case::{Case::Camel, Casing};
 use proc_macro::TokenStream;
 use proc_macro2::{Ident, Span};
 use proc_macro_error::{abort, abort_call_site, proc_macro_error};
 use quote::quote;
 use regex::Regex;
+use serde_derive::Deserialize;
 use tiny_keccak::{Hasher, Keccak};
+
+type MayString = Option<String>;
+
+#[derive(Deserialize)]
+#[serde(rename_all = "camelCase")]
+#[allow(dead_code)]
+struct AbiIO {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    internal_type: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    name: MayString,
+    r#type: String,
+}
+
 fn get_function_signature(function_prototype: &str) -> [u8; 4] {
     let mut sig = [0; 4];
     let mut hasher = Keccak::v256();
@@ -16,10 +35,12 @@ fn get_function_signature(function_prototype: &str) -> [u8; 4] {
 }
 
 fn write_function_signature(sig_str: &str) -> String {
-    let re = Regex::new(r"^(?P<name>[^(]+?)\((?P<params>[^)]*?)\)").unwrap();
+    let re = unsafe { Regex::new(r"^(?P<name>[^(]+?)\((?P<params>[^)]*?)\)").unwrap_unchecked() };
     if let Some(cap) = re.captures(sig_str) {
-        let fn_name = cap.name("name").unwrap().as_str();
-        let params = cap.name("params").unwrap().as_str().replace(" ", "");
+        let fn_name = unsafe { cap.name("name").unwrap_unchecked() }.as_str();
+        let params = unsafe { cap.name("params").unwrap_unchecked() }
+            .as_str()
+            .replace(" ", "");
         let canonical_fn = format!(
             "{}({})",
             fn_name,
@@ -239,6 +260,107 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
     }.into()
 }
 
+fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, String), &'static str> {
+    let attr_str = attr.replace(" ", "").replace("\n", "");
+    return if attr_str.is_empty() {
+        Ok((None, "{}".into()))
+    } else if let Some((head, tail)) = attr_str.split_once(',') {
+        if tail.is_empty() {
+            Ok((Some(head.replace("\"", "")), "{}".into()))
+        } else {
+            let mut json = "{".to_string();
+            if let Some(cap) =
+                unsafe { Regex::new(r"constant=(?P<constant>[^,]*)").unwrap_unchecked() }
+                    .captures(&attr_str)
+            {
+                match unsafe { cap.name("constant").unwrap_unchecked() }.as_str() {
+                    "true" => json.push_str(r#""constant":true,"#),
+                    "false" => json.push_str(r#""constant":false,"#),
+                    _ => return Err("constacnt should be true or false"),
+                }
+            } else {
+                json.push_str(r#""constant":false,"#)
+            }
+
+            if let Some(cap) =
+                unsafe { Regex::new(r"inputs=(?P<inputs>\[[^\[\]]*\])").unwrap_unchecked() }
+                    .captures(&attr_str)
+            {
+                json.push_str(r#""inputs":"#);
+                let inputs_str = unsafe { cap.name("inputs").unwrap_unchecked() }.as_str();
+                for input_cap in Regex::new(r"\{[^\{\}]*\}").unwrap().find_iter(inputs_str) {
+                    if serde_json::from_str::<AbiIO>(input_cap.as_str()).is_err() {
+                        return Err("inputs are not valid format");
+                    }
+                }
+                json.push_str(inputs_str);
+                json.push(',');
+            } else {
+                json.push_str(r#""inputs":[],"#);
+            }
+
+            if let Some(cap) = unsafe { Regex::new(r"name=(?P<name>[^,]*)").unwrap_unchecked() }
+                .captures(&attr_str)
+            {
+                json.push_str(r#""name":""#);
+                json.push_str(unsafe { cap.name("name").unwrap_unchecked() }.as_str());
+                json.push_str(r#"","#);
+            } else {
+                json.push_str(&format!(r#""name":"{}","#, fn_name.to_case(Camel)));
+            }
+
+            if let Some(cap) =
+                unsafe { Regex::new(r"outputs=(?P<outputs>\[[^\[\]]*\])").unwrap_unchecked() }
+                    .captures(&attr_str)
+            {
+                json.push_str(r#""outputs":"#);
+                let outputs_str = unsafe { cap.name("outputs").unwrap_unchecked() }.as_str();
+                for output_cap in Regex::new(r"\{[^\{\}]*\}").unwrap().find_iter(outputs_str) {
+                    if serde_json::from_str::<AbiIO>(output_cap.as_str()).is_err() {
+                        return Err("outputs are not valid format");
+                    }
+                }
+                json.push_str(outputs_str);
+                json.push(',');
+            } else {
+                json.push_str(r#""outputs":[],"#);
+            }
+
+            if let Some(cap) =
+                unsafe { Regex::new(r"payable=(?P<payable>[^,]*)").unwrap_unchecked() }
+                    .captures(&attr_str)
+            {
+                match unsafe { cap.name("payable").unwrap_unchecked() }.as_str() {
+                    "true" => json.push_str(r#""payable":true,"#),
+                    "false" => json.push_str(r#""payable":false,"#),
+                    _ => return Err("payable should be true or false"),
+                }
+            } else {
+                json.push_str(r#""payable":false,"#);
+            }
+
+            if let Some(cap) = unsafe {
+                Regex::new(r"stateMutability=(?P<stateMutability>[^,]*)").unwrap_unchecked()
+            }
+            .captures(&attr_str)
+            {
+                match unsafe { cap.name("stateMutability").unwrap_unchecked() }.as_str() {
+                    "nonpayable" => json.push_str(r#""stateMutability":"nonpayable","#),
+                    "view" => json.push_str(r#""stateMutability":"view","#),
+                    _ => return Err("stateMutability should be nonpayable or view"),
+                }
+            } else {
+                json.push_str(r#""stateMutability":"view","#);
+            }
+
+            json.push_str(r#""type":"function"}"#);
+            Ok((Some(head.replace("\"", "")), json))
+        }
+    } else {
+        Ok((Some(attr_str.replace("\"", "")), "{}".into()))
+    };
+}
+
 /// helps you to build your handlers in the contract
 ///
 /// This macro also generate the function signature, you can use
@@ -261,25 +383,41 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
 /// }
 /// ```
 ///
+/// There are two kind of inputs for `ewasm_fn` macro, first is functional signature, second and
+/// more are the fields of abijson.
+/// The functional signature can be specific as following ways.
+/// `#[ewasm_fn(00fdd58e)]` or #[ewasm_fn("00fdd58e")]
+///
+/// ``` #[ewasm_fn(00fdd58e,
+/// constant=true,
+/// inputs=[
+///     { "internalType": "address", "name": "account", "type": "address" },
+///     { "internalType": "uinit256", "name": "token_id", "type": "uinit256" }
+/// ],
+/// name=balanceOf,
+/// outputs=[
+///     { "internalType": "uint256", "name": "", "type": "uint256" }
+/// ],
+/// payable=false,
+/// stateMutability=view
+/// )]
+/// ```
+/// The fields are not required, it can use default value if not provided.
+/// The default values of `constant`, `payable` are `false`; the default values of `inputs` and
+/// `outputs` are `[]`; the default value of `stateMutability` is `view`; the default name is the
+/// camel case style of the function name.
+///
 #[proc_macro_error]
 #[proc_macro_attribute]
 pub fn ewasm_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
-    let attr_str = attr.to_string().replace(" ", "");
-    let (hex_str, abi_str) = if attr_str.is_empty() {
-        (None, "{}".to_string())
-    } else if attr_str.starts_with('{') {
-        (None, attr_str.split_whitespace().collect())
-    } else if let Some((head, tail)) = attr_str.split_once(',') {
-        (
-            Some(head.replace("\"", "")),
-            tail.split_whitespace().collect(),
-        )
-    } else {
-        (Some(attr_str.replace("\"", "")), "{}".to_string())
-    };
-
     let input = syn::parse_macro_input!(item as syn::ItemFn);
     let name = &input.sig.ident;
+
+    let (hex_str, abi_str) = match parse_fn_attr(name.to_string(), attr.to_string()) {
+        Ok(o) => o,
+        Err(e) => abort_call_site!(e),
+    };
+
     let args = &input
         .sig
         .inputs
@@ -291,9 +429,25 @@ pub fn ewasm_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
             syn::FnArg::Typed(p) => Box::into_inner(p.ty.clone()),
         })
         .map(|ty| match ty {
-            syn::Type::Path(tp) => (tp.path.segments.first().unwrap().ident.clone(), false),
+            syn::Type::Path(tp) => (
+                tp.path
+                    .segments
+                    .first()
+                    .expect("at least one segment")
+                    .ident
+                    .clone(),
+                false,
+            ),
             syn::Type::Reference(tr) => match Box::into_inner(tr.elem) {
-                syn::Type::Path(tp) => (tp.path.segments.first().unwrap().ident.clone(), true),
+                syn::Type::Path(tp) => (
+                    tp.path
+                        .segments
+                        .first()
+                        .expect("at least one segment")
+                        .ident
+                        .clone(),
+                    true,
+                ),
                 _ => abort_call_site!("please pass Path type or Reference type to ewasm_fn_sig"),
             },
             _ => abort_call_site!("please pass Path type or Reference type to ewasm_fn_sig"),
@@ -390,22 +544,42 @@ pub fn ewasm_constructor(_attr: TokenStream, item: TokenStream) -> TokenStream {
 ///     Ok(())
 /// }
 /// ```
+///
+/// There are two kind of inputs for `ewasm_fn` macro, first is functional signature, second and
+/// more are the fields of abijson.
+/// The functional signature can be specific as following ways.
+/// `#[ewasm_lib_fn(00fdd58e)]` or #[ewasm_lib_fn("00fdd58e")]
+///
+/// ``` #[ewasm_lib_fn(00fdd58e,
+/// constant=true,
+/// inputs=[
+///     { "internalType": "address", "name": "account", "type": "address" },
+///     { "internalType": "uinit256", "name": "token_id", "type": "uinit256" }
+/// ],
+/// name=balanceOf,
+/// outputs=[
+///     { "internalType": "uint256", "name": "", "type": "uint256" }
+/// ],
+/// payable=false,
+/// stateMutability=view
+/// )]
+/// ```
+/// The fields are not required, it can use default value if not provided.
+/// The default values of `constant`, `payable` are `false`; the default values of `inputs` and
+/// `outputs` are `[]`; the default value of `stateMutability` is `view`; the default name is the
+/// camel case style of the function name.
+///
 #[proc_macro_error]
 #[proc_macro_attribute]
 pub fn ewasm_lib_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
-    let attr_str = attr.to_string().replace(" ", "");
-    let (hex_str, abi_str) = if attr_str.is_empty() {
-        (None, "{}".to_string())
-    } else if attr_str.starts_with('{') {
-        (None, attr_str)
-    } else if let Some((head, tail)) = attr_str.split_once(',') {
-        (Some(head.replace("\"", "")), tail.to_string())
-    } else {
-        (Some(attr_str.replace("\"", "")), "{}".to_string())
-    };
-
     let input = syn::parse_macro_input!(item as syn::ItemFn);
     let name = &input.sig.ident;
+
+    let (hex_str, abi_str) = match parse_fn_attr(name.to_string(), attr.to_string()) {
+        Ok(o) => o,
+        Err(e) => abort_call_site!(e),
+    };
+
     let inputs = &input.sig.inputs;
 
     let (sig_0, sig_1, sig_2, sig_3) = if let Some(hex_str) = hex_str {
@@ -421,9 +595,25 @@ pub fn ewasm_lib_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
                 syn::FnArg::Typed(p) => Box::into_inner(p.ty.clone()),
             })
             .map(|ty| match ty {
-                syn::Type::Path(tp) => (tp.path.segments.first().unwrap().ident.clone(), false),
+                syn::Type::Path(tp) => (
+                    tp.path
+                        .segments
+                        .first()
+                        .expect("at least one segment")
+                        .ident
+                        .clone(),
+                    false,
+                ),
                 syn::Type::Reference(tr) => match Box::into_inner(tr.elem) {
-                    syn::Type::Path(tp) => (tp.path.segments.first().unwrap().ident.clone(), true),
+                    syn::Type::Path(tp) => (
+                        tp.path
+                            .segments
+                            .first()
+                            .expect("at least one segment")
+                            .ident
+                            .clone(),
+                        true,
+                    ),
                     _ => {
                         abort_call_site!("please pass Path type or Reference type to ewasm_fn_sig")
                     }
@@ -517,7 +707,9 @@ pub fn ewasm_lib_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_fn_sig(item: TokenStream) -> TokenStream {
-    write_function_signature(&item.to_string()).parse().unwrap()
+    write_function_signature(&item.to_string())
+        .parse()
+        .expect("can not generate function signature")
 }
 
 /// helps you generate the input raw data for specific contract handler
@@ -527,10 +719,10 @@ pub fn ewasm_fn_sig(item: TokenStream) -> TokenStream {
 /// ```
 #[proc_macro]
 pub fn ewasm_input(item: TokenStream) -> TokenStream {
-    let re = Regex::new(r"(?P<instance>.*)\s+for\s+(?P<sig>.*)").unwrap();
+    let re = unsafe { Regex::new(r"(?P<instance>.*)\s+for\s+(?P<sig>.*)").unwrap_unchecked() };
     if let Some(cap) = re.captures(&item.to_string()) {
-        let sig = cap.name("sig").unwrap().as_str();
-        let instance = cap.name("instance").unwrap().as_str();
+        let sig = unsafe { cap.name("sig").unwrap_unchecked() }.as_str();
+        let instance = unsafe { cap.name("instance").unwrap_unchecked() }.as_str();
         let output = if instance == "None" {
             format!("{}.to_vec()", write_function_signature(sig),)
         } else {
@@ -544,7 +736,7 @@ pub fn ewasm_input(item: TokenStream) -> TokenStream {
                 instance
             )
         };
-        output.parse().unwrap()
+        output.parse().expect("ewasm input transform unexpected")
     } else {
         panic!("ewasm_input input incorrect")
     }
@@ -583,21 +775,26 @@ pub fn ewasm_input(item: TokenStream) -> TokenStream {
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_input_from(item: TokenStream) -> TokenStream {
-    let re =
-        Regex::new(r"^(?P<contract>\w+)\s+move\s+(?P<name>[^,]+),?(?P<error_handler>.*)").unwrap();
+    let re = unsafe {
+        Regex::new(r"^(?P<contract>\w+)\s+move\s+(?P<name>[^,]+),?(?P<error_handler>.*)")
+            .unwrap_unchecked()
+    };
     if let Some(cap) = re.captures(&item.to_string()) {
-        let contract = Ident::new(cap.name("contract").unwrap().as_str(), Span::call_site());
+        let contract = Ident::new(
+            unsafe { cap.name("contract").unwrap_unchecked() }.as_str(),
+            Span::call_site(),
+        );
         let name_result: syn::Result<syn::ExprPath> =
-            syn::parse_str(cap.name("name").unwrap().as_str());
+            syn::parse_str(unsafe { cap.name("name").unwrap_unchecked() }.as_str());
         let name = if let Ok(name) = name_result {
             name
         } else {
             abort_call_site!(
                 "`{}` is not an ExprPath",
-                cap.name("name").unwrap().as_str()
+                unsafe { cap.name("name").unwrap_unchecked() }.as_str()
             );
         };
-        let error_handler = cap.name("error_handler").unwrap().as_str();
+        let error_handler = unsafe { cap.name("error_handler").unwrap_unchecked() }.as_str();
         return if error_handler.is_empty() {
             quote! {
                 #name(sewup::bincode::deserialize(&#contract.input_data[4..])
@@ -635,7 +832,7 @@ pub fn ewasm_output_from(item: TokenStream) -> TokenStream {
         item,
     )
     .parse()
-    .unwrap()
+    .expect("ewasm output format unexpected")
 }
 
 /// `Key` derive help you implement Key trait for the kv feature
@@ -740,12 +937,44 @@ pub fn derive_value(item: TokenStream) -> TokenStream {
 /// let default_person_input: person::Protocol = Person::default().into();
 /// assert!(default_input != default_person_input)
 /// ```
+///
+/// The relation can be set with `#[belongs_to(Person)]` or `#[belongs_none_or(Person)]`
+/// ```compile_fail
+/// #[derive(Table, Default, Clone, PartialEq, Serialize, Deserialize)]
+/// #[belongs_none_or(Location)]
+/// pub struct Person {
+///     pub trusted: bool,
+///     pub age: u8,
+///     pub location_id: Option<usize>,
+/// }
+///
+/// #[derive(Table, Default, Clone, PartialEq, Serialize, Deserialize)]
+/// #[belongs_to(Person)]
+/// pub struct Post {
+///     pub content: SizedString!(50),
+///
+///     // Currently, this field need to set up manually, this will be enhance later
+///     pub person_id: usize,
+/// }
+///
+/// #[derive(Table, Default, Clone, PartialEq, Serialize, Deserialize)]
+/// pub struct Location {
+///     pub address: Raw,
+/// }
+/// ```
+/// After the directive setup, you can use `table_name.another_table_name` to get the related
+/// instance, for example
+/// ```compile_fail
+/// let person: Person = post.person()?;
+/// let home: Option<Location> = person.location()?;
+/// ```
 #[cfg(feature = "rdb")]
 #[proc_macro_derive(Table, attributes(belongs_to, belongs_none_or))]
 pub fn derive_table(item: TokenStream) -> TokenStream {
     let input = syn::parse_macro_input!(item as syn::DeriveInput);
     let attrs = &input.attrs;
     let mut belongs_to: Option<String> = None;
+    let mut belongs_none_or: Option<String> = None;
     for a in attrs.iter() {
         let syn::Attribute { path, tokens, .. } = a;
         let attr_name = path.segments.first().map(|s| s.ident.to_string());
@@ -760,6 +989,17 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
                     .to_string(),
             );
         }
+        if Some("belongs_none_or".to_string()) == attr_name {
+            belongs_none_or = Some(
+                tokens
+                    .to_string()
+                    .strip_prefix('(')
+                    .expect("#[belongs_none_or(table_name)] is not correct")
+                    .strip_suffix(')')
+                    .expect("#[belongs_none_or(table_name)] is not correct")
+                    .to_string(),
+            );
+        }
     }
     let struct_name = &input.ident;
     let fields_with_type = match &input.data {
@@ -1066,6 +1306,31 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
         .to_string();
     }
 
+    if let Some(parent_table) = belongs_none_or {
+        let lower_parent_table = &format!("{}", &parent_table).to_ascii_lowercase();
+        let parent_table = Ident::new(&parent_table, Span::call_site());
+        let lower_parent_table_ident = Ident::new(&lower_parent_table, Span::call_site());
+        let field_name = &format!("{}_id", lower_parent_table);
+
+        output += &quote! {
+            impl #struct_name {
+                pub fn #lower_parent_table_ident (&self) -> sewup::Result<Option<#parent_table>> {
+                    let id: Option<usize> = sewup::utils::get_field_by_name(self, #field_name);
+                    let parent_table = sewup::rdb::Db::load(None)?.table::<#parent_table>()?;
+                    if let Some(id) = id {
+                        match parent_table.get_record(id) {
+                            Ok(r) => Ok(Some(r)),
+                            Err(e) => Err(e)
+                        }
+                    } else {
+                        Ok(None)
+                    }
+                }
+            }
+        }
+        .to_string();
+    }
+
     output.parse().unwrap()
 }
 
@@ -1099,8 +1364,12 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
 #[proc_macro_error]
 #[proc_macro_attribute]
 pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
-    let mod_re = Regex::new(r"^mod (?P<mod_name>[^\{\s]*)(?P<to_first_bracket>[^\{]*\{)").unwrap();
-    let fn_re = Regex::new(r"^fn (?P<fn_name>[^\(\s]*)(?P<to_first_bracket>[^\{]*\{)").unwrap();
+    let mod_re = unsafe {
+        Regex::new(r"^mod (?P<mod_name>[^\{\s]*)(?P<to_first_bracket>[^\{]*\{)").unwrap_unchecked()
+    };
+    let fn_re = unsafe {
+        Regex::new(r"^fn (?P<fn_name>[^\(\s]*)(?P<to_first_bracket>[^\{]*\{)").unwrap_unchecked()
+    };
     let context = item.to_string();
     if mod_re.captures(&context).is_some() {
         let attr_str = attr.to_string().replace(" ", "");
@@ -1262,11 +1531,17 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_assert_eq(item: TokenStream) -> TokenStream {
-    let re = Regex::new(r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*,(?P<equivalence>.*)"#).unwrap();
+    let re = unsafe {
+        Regex::new(r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*,(?P<equivalence>.*)"#).unwrap_unchecked()
+    };
     if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
-        let fn_name = cap.name("fn_name").unwrap().as_str().replace(" ", "");
-        let params = cap.name("params").unwrap().as_str().replace(" ", "");
-        let equivalence = cap.name("equivalence").unwrap().as_str();
+        let fn_name = unsafe { cap.name("fn_name").unwrap_unchecked() }
+            .as_str()
+            .replace(" ", "");
+        let params = unsafe { cap.name("params").unwrap_unchecked() }
+            .as_str()
+            .replace(" ", "");
+        let equivalence = unsafe { cap.name("equivalence").unwrap_unchecked() }.as_str();
         let caller = cap
             .name("caller")
             .map(|c| format!("Some({})", c.as_str()))
@@ -1300,11 +1575,15 @@ pub fn ewasm_assert_eq(item: TokenStream) -> TokenStream {
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_auto_assert_eq(item: TokenStream) -> TokenStream {
-    let re = Regex::new(r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*,(?P<equivalence>.*)"#).unwrap();
+    let re = unsafe {
+        Regex::new(r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*,(?P<equivalence>.*)"#).unwrap_unchecked()
+    };
     if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
-        let fn_name = cap.name("fn_name").unwrap().as_str();
-        let params = cap.name("params").unwrap().as_str().replace(" ", "");
-        let equivalence = cap.name("equivalence").unwrap().as_str();
+        let fn_name = unsafe { cap.name("fn_name").unwrap_unchecked() }.as_str();
+        let params = unsafe { cap.name("params").unwrap_unchecked() }
+            .as_str()
+            .replace(" ", "");
+        let equivalence = unsafe { cap.name("equivalence").unwrap_unchecked() }.as_str();
         let caller = cap
             .name("caller")
             .map(|c| format!("Some({})", c.as_str()))
@@ -1351,13 +1630,17 @@ pub fn ewasm_auto_assert_eq(item: TokenStream) -> TokenStream {
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_assert_ok(item: TokenStream) -> TokenStream {
-    let re = Regex::new(
-        r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*"#,
-    )
-    .unwrap();
+    let re = unsafe {
+        Regex::new(
+            r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*"#,
+        )
+        .unwrap_unchecked()
+    };
     if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
-        let fn_name = cap.name("fn_name").unwrap().as_str();
-        let params = cap.name("params").unwrap().as_str().replace(" ", "");
+        let fn_name = unsafe { cap.name("fn_name").unwrap_unchecked() }.as_str();
+        let params = unsafe { cap.name("params").unwrap_unchecked() }
+            .as_str()
+            .replace(" ", "");
         let caller = cap
             .name("caller")
             .map(|c| format!("Some({})", c.as_str()))
@@ -1391,13 +1674,17 @@ pub fn ewasm_assert_ok(item: TokenStream) -> TokenStream {
 #[proc_macro_error]
 #[proc_macro]
 pub fn ewasm_rusty_assert_ok(item: TokenStream) -> TokenStream {
-    let re = Regex::new(
-        r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*"#,
-    )
-    .unwrap();
+    let re = unsafe {
+        Regex::new(
+            r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*"#,
+        )
+        .unwrap_unchecked()
+    };
     if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
-        let fn_name = cap.name("fn_name").unwrap().as_str();
-        let params = cap.name("params").unwrap().as_str().replace(" ", "");
+        let fn_name = unsafe { cap.name("fn_name").unwrap_unchecked() }.as_str();
+        let params = unsafe { cap.name("params").unwrap_unchecked() }
+            .as_str()
+            .replace(" ", "");
         let caller = cap
             .name("caller")
             .map(|c| format!("Some({})", c.as_str()))
